/*
 * Copyright (c) 2006 Suguru ARAKAWA. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package jp.ac.hosei.k.cis.java2005.dita.swing;

import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.text.MessageFormat;

import javax.swing.AbstractAction;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;

import jp.ac.hosei.k.cis.java2005.dita.BinaryEditorController;
import jp.ac.hosei.k.cis.java2005.dita.BinaryEditorView;

/**
 * {@link BinaryEditorView} の Swing による実装。
 * @author arakawa
 * @version $Id: BinaryEditorSwingView.java,v 1.1 2005/11/17 05:14:12 java2005 Exp $
 */
public class BinaryEditorSwingView implements BinaryEditorView {
    
    private final BinaryEditorController controler;

    private JFrame frame;

    private BinaryEditorPane editor;

    private File current;

    /**
     * Swing で表現されたビューを作成する。
     * @param controler コントローラインスタンス
     * @throws NullPointerException 引数に<code>null</code>が含まれていた場合
     */
    public BinaryEditorSwingView(BinaryEditorController controler) {
        super();
        if (controler == null) {
            throw new NullPointerException("controler"); //$NON-NLS-1$
        }
        this.controler = controler;
        this.current = null;
        this.init();
    }

    /**
     * このビューに、ファイルを開く操作を要求する。
     * @return ビューがユーザから取得した、開くべきファイル
     */
    public File requestFileOpen() {
        JFileChooser chooser = createFileChooser(); 
        chooser.showOpenDialog(this.frame);
        return chooser.getSelectedFile();
    }

    /**
     * このビューに、ファイルを保存する操作を要求する。
     * @return ビューがユーザから取得した、保存すべき対象のファイル
     */
    public File requestFileSave() {
        JFileChooser chooser = createFileChooser(); 
        chooser.showSaveDialog(this.frame);
        return chooser.getSelectedFile();
    }
    
    /**
     * {@link JFileChooser} を作成する。
     * @return {@link JFileChooser} のインスタンス
     */
    private JFileChooser createFileChooser() {
        File parent;
        if (this.current == null) {
            parent = new File("."); //$NON-NLS-1$
        }
        else {
            parent = this.current.getParentFile();
        }
        return new JFileChooser(parent);
    }

    /**
     * このビューに、プログラムを終了する操作を要求する。
     * @return ユーザが終了を選択した場合は <code>true</code>
     */
    public boolean requestExit() {
        int result = JOptionPane.showConfirmDialog(this.frame.getContentPane(), Messages.getString("BinaryEditorSwingView.confirm_exit_message"), Messages.getString("BinaryEditorSwingView.confirm_exit_title"), //$NON-NLS-1$ //$NON-NLS-2$
            JOptionPane.YES_NO_OPTION);
        if (result == JOptionPane.YES_OPTION) {
            this.frame.dispose();
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * このビューに、指定したファイルを読み込んだことを通知する。
     * @param file 読み込んだファイル
     * @throws NullPointerException 引数に<code>null</code>が含まれていた場合
     */
    public void notifyFileOpened(File file) {
        if (file == null) {
            throw new NullPointerException("file"); //$NON-NLS-1$
        }
        changeCurrentFile(file);
    }

    /**
     * このビューに、指定したファイルにデータを保存したことを通知する。
     * @param file 保存したファイル
     * @throws NullPointerException 引数に<code>null</code>が含まれていた場合
     */
    public void notifyFileSaved(File file) {
        if (file == null) {
            throw new NullPointerException("file"); //$NON-NLS-1$
        }
        JOptionPane.showMessageDialog(
            this.frame.getContentPane(),
            MessageFormat.format(
                Messages.getString("BinaryEditorSwingView.info_saved_message"), //$NON-NLS-1$
                new Object[]{file.getAbsolutePath()}),
            Messages.getString("BinaryEditorSwingView.info_saved_title"), JOptionPane.INFORMATION_MESSAGE); //$NON-NLS-1$
        changeCurrentFile(file);
    }

    /**
     * 現在扱っているファイルの名前をビューに反映させる。
     * @param file 扱っているファイル
     * @throws NullPointerException 引数に<code>null</code>が含まれていた場合
     */
    private void changeCurrentFile(File file) {
        if (file == null) {
            throw new NullPointerException("file"); //$NON-NLS-1$
        }
        this.current = file;
        this.frame.setTitle(MessageFormat.format(
            Messages.getString("BinaryEditorSwingView.window_title"), //$NON-NLS-1$
            new Object[]{file.getAbsolutePath()}
            ));
    }

    /**
     * このビューに、エラーが発生したことを通知する。
     * @param summary エラーの概要
     * @param message エラーメッセージ
     * @throws NullPointerException 引数に<code>null</code>が含まれていた場合
     */
    public void notifyErrorOccurred(String summary, String message) {
        if (summary == null) {
            throw new NullPointerException("summary"); //$NON-NLS-1$
        }
        if (message == null) {
            throw new NullPointerException("message"); //$NON-NLS-1$
        }
        JOptionPane.showMessageDialog(this.frame.getContentPane(), message, summary, JOptionPane.ERROR_MESSAGE);
    }

    /**
     * このビューのデータを変更する。
     * @param data 変更先のデータ
     * @throws NullPointerException 引数に<code>null</code>が含まれていた場合
     */
    public void setData(byte[] data) {
        if (data == null) {
            throw new NullPointerException("data"); //$NON-NLS-1$
        }
        this.editor.setData(data);
    }

    /**
     * このビューが持つデータを取得する。
     * @return 現在のデータ
     */
    public byte[] getData() {
        return this.editor.getData();
    }

    /**
     * フレームを初期化する。
     */
    private void init() {
        this.frame = new JFrame(Messages.getString("BinaryEditorSwingView.init_window_title")); //$NON-NLS-1$
        final BinaryEditorController ctrl = this.controler;

        // window
        this.frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                ctrl.performExit();
            }
        });

        this.frame.setJMenuBar(prepareMenu());

        this.editor = new BinaryEditorPane();
        this.frame.getContentPane().add(new JScrollPane(this.editor));

        this.frame.setSize(500, 300);
        this.frame.setVisible(true);
    }

    /**
     * メニューバーを作成する。
     * @return 新しいメニューバー
     */
    private JMenuBar prepareMenu() {
        final BinaryEditorController ctrl = this.controler;
        
        // open
        JMenuItem open = new JMenuItem(new AbstractAction(Messages.getString("BinaryEditorSwingView.open_button")) { //$NON-NLS-1$
            private static final long serialVersionUID = 1L;
            public void actionPerformed(ActionEvent e) {
                ctrl.performFileOpen();
            }
        });
        open.setMnemonic('O');

        // save
        JMenuItem save = new JMenuItem(new AbstractAction(Messages.getString("BinaryEditorSwingView.save_button")) { //$NON-NLS-1$
            private static final long serialVersionUID = 1L;
            public void actionPerformed(ActionEvent e) {
                ctrl.performFileSave();
            }
        });
        save.setMnemonic('S');

        // exit
        JMenuItem exit = new JMenuItem(new AbstractAction(Messages.getString("BinaryEditorSwingView.exit_button")) { //$NON-NLS-1$
            private static final long serialVersionUID = 1L;
            public void actionPerformed(ActionEvent e) {
                ctrl.performExit();
            }
        });
        exit.setMnemonic('x');

        // file
        JMenu menuFile = new JMenu(Messages.getString("BinaryEditorSwingView.file_menu")); //$NON-NLS-1$
        menuFile.setMnemonic('F');
        menuFile.add(open);
        menuFile.add(save);
        menuFile.addSeparator();
        menuFile.add(exit);

        JMenuBar menu = new JMenuBar();
        menu.add(menuFile);

        return menu;
    }
}
